//================================================================================
// m2v_dec_eval - VLC テーブル
// $Id: vlc.c 58 2010-11-25 07:17:55Z aberi $
//================================================================================

#include "vlc.h"
#include <malloc.h>

// Macroblock addressing
// checked (mpeg12data.c)
static const VLC_ENTRY vlc_entries_b1[] =
{
	{ 0b1,           1,  1  },
	{ 0b011,         3,  2  },
	{ 0b010,         3,  3  },
	{ 0b0011,        4,  4  },
	{ 0b0010,        4,  5  },
	{ 0b00011,       5,  6  },
	{ 0b00010,       5,  7  },
	{ 0b0000111,     7,  8  },
	{ 0b0000110,     7,  9  },
	{ 0b00001011,    8,  10 },
	{ 0b00001010,    8,  11 },
	{ 0b00001001,    8,  12 },
	{ 0b00001000,    8,  13 },
	{ 0b00000000,    8,  0  },	// end (and 15 more 0 bits should follow)
	{ 0b00000111,    8,  14 },
	{ 0b00000110,    8,  15 },
	{ 0b0000010111,  10, 16 },
	{ 0b0000010110,  10, 17 },
	{ 0b0000010101,  10, 18 },
	{ 0b0000010100,  10, 19 },
	{ 0b0000010011,  10, 20 },
	{ 0b0000010010,  10, 21 },
	{ 0b00000100011, 11, 22 },
	{ 0b00000100010, 11, 23 },
	{ 0b00000100001, 11, 24 },
	{ 0b00000100000, 11, 25 },
	{ 0b00000011111, 11, 26 },
	{ 0b00000011110, 11, 27 },
	{ 0b00000011101, 11, 28 },
	{ 0b00000011100, 11, 29 },
	{ 0b00000011011, 11, 30 },
	{ 0b00000011010, 11, 31 },
	{ 0b00000011001, 11, 32 },
	{ 0b00000011000, 11, 33 },
	{ 0b00000001000, 11, -1 },	// escape
//	{ 0b00000001111, 11, -1 },	// not available @ ISO13818-2
	{ 0, 0, 1 },
};

// for I pictures
static const VLC_ENTRY vlc_entries_b2[] =
{
	//           +--------Quant
	//           |+-------MoFw
	//           ||+------MoBw
	//           |||+-----Pat
	//           ||||+----Intra
	//           |||||+---STWF
	//           ||||||+--PermittedSTW(1,2,3)
	//           |||||||+-PermittedSTW(4)
	//           ||||||||
	{ 0b1,  1, 0b00001000 },
	{ 0b01, 2, 0b10001000 },
	{ 0, 0, 2 },
};

// for P pictures
static const VLC_ENTRY vlc_entries_b3[] =
{
	//               +--------Quant
	//               |+-------MoFw
	//               ||+------MoBw
	//               |||+-----Pat
	//               ||||+----Intra
	//               |||||+---STWF
	//               ||||||+--PermittedSTW(1,2,3)
	//               |||||||+-PermittedSTW(4)
	//               ||||||||
	{ 0b1,      1, 0b01010000 },
	{ 0b01,     2, 0b00010000 },
	{ 0b001,    3, 0b01000000 },
	{ 0b00011,  5, 0b00001000 },
	{ 0b00010,  5, 0b11010000 },
	{ 0b00001,  5, 0b10010000 },
	{ 0b000001, 6, 0b10001000 },
	{ 0, 0, 3 },
};

// for B pictures
static const VLC_ENTRY vlc_entries_b4[] =
{
	//               +--------Quant
	//               |+-------MoFw
	//               ||+------MoBw
	//               |||+-----Pat
	//               ||||+----Intra
	//               |||||+---STWF
	//               ||||||+--PermittedSTW(1,2,3)
	//               |||||||+-PermittedSTW(4)
	//               ||||||||
	{ 0b10,     2, 0b01100000 },
	{ 0b11,     2, 0b01110000 },
	{ 0b010,    3, 0b00100000 },
	{ 0b011,    3, 0b00110000 },
	{ 0b0010,   4, 0b01000000 },
	{ 0b0011,   4, 0b01010000 },
	{ 0b00011,  5, 0b00001000 },
	{ 0b00010,  5, 0b11110000 },
	{ 0b000011, 6, 0b11010000 },
	{ 0b000010, 6, 0b10110000 },
	{ 0b000001, 6, 0b10001000 },
	{ 0, 0, 4 },
};

// for I pictures with spatial scalability
// static const VLC_ENTRY vlc_entries_b5[] =

// for P pictures with spatial scalability
// static const VLC_ENTRY vlc_entries_b6[] =

// for B pictures with spatial scalability
// static const VLC_ENTRY vlc_entries_b7[] =

// for I,P,B pictures with SNR scalability
// static const VLC_ENTRY vlc_entries_b8[] =

// coded block pattern
// checked (mpeg12data.c)
static const VLC_ENTRY vlc_entries_b9[] =
{
	{ 0b111,       3, 60 },
	{ 0b1101,      4,  4 },
	{ 0b1100,      4,  8 },
	{ 0b1011,      4, 16 },
	{ 0b1010,      4, 32 },
	{ 0b10011,     5, 12 },
	{ 0b10010,     5, 48 },
	{ 0b10001,     5, 20 },
	{ 0b10000,     5, 40 },
	{ 0b01111,     5, 28 },
	{ 0b01110,     5, 44 },
	{ 0b01101,     5, 52 },
	{ 0b01100,     5, 56 },
	{ 0b01011,     5,  1 },
	{ 0b01010,     5, 61 },
	{ 0b01001,     5,  2 },
	{ 0b01000,     5, 62 },
	{ 0b001111,    6, 24 },
	{ 0b001110,    6, 36 },
	{ 0b001101,    6,  3 },
	{ 0b001100,    6, 63 },
	{ 0b0010111,   7,  5 },
	{ 0b0010110,   7,  9 },
	{ 0b0010101,   7, 17 },
	{ 0b0010100,   7, 33 },
	{ 0b0010011,   7,  6 },
	{ 0b0010010,   7, 10 },
	{ 0b0010001,   7, 18 },
	{ 0b0010000,   7, 34 },
	{ 0b00011111,  8,  7 },
	{ 0b00011110,  8, 11 },
	{ 0b00011101,  8, 19 },
	{ 0b00011100,  8, 35 },
	{ 0b00011011,  8, 13 },
	{ 0b00011010,  8, 49 },
	{ 0b00011001,  8, 21 },
	{ 0b00011000,  8, 41 },
	{ 0b00010111,  8, 14 },
	{ 0b00010110,  8, 50 },
	{ 0b00010101,  8, 22 },
	{ 0b00010100,  8, 42 },
	{ 0b00010011,  8, 15 },
	{ 0b00010010,  8, 51 },
	{ 0b00010001,  8, 23 },
	{ 0b00010000,  8, 43 },
	{ 0b00001111,  8, 25 },
	{ 0b00001110,  8, 37 },
	{ 0b00001101,  8, 26 },
	{ 0b00001100,  8, 38 },
	{ 0b00001011,  8, 29 },
	{ 0b00001010,  8, 45 },
	{ 0b00001001,  8, 53 },
	{ 0b00001000,  8, 57 },
	{ 0b00000111,  8, 30 },
	{ 0b00000110,  8, 46 },
	{ 0b00000101,  8, 54 },
	{ 0b00000100,  8, 58 },
	{ 0b000000111, 9, 31 },
	{ 0b000000110, 9, 47 },
	{ 0b000000101, 9, 55 },
	{ 0b000000100, 9, 59 },
	{ 0b000000011, 9, 27 },
	{ 0b000000010, 9, 39 },
	{ 0b000000001, 9,  0 },
	{ 0, 0, 9 },
};

// motion_code
static const VLC_ENTRY vlc_entries_b10[] =
{
	{ 0b1,           1,   0  },
	{ 0b011,         3,  -1  },
	{ 0b010,         3,   1  },
	{ 0b0011,        4,  -2  },
	{ 0b0010,        4,   2  },
	{ 0b00011,       5,  -3  },
	{ 0b00010,       5,   3  },
	{ 0b0000111,     7,  -4  },
	{ 0b0000110,     7,   4  },
	{ 0b00000111,    8,  -7  },
	{ 0b00001001,    8,  -6  },
	{ 0b00001011,    8,  -5  },
	{ 0b00001010,    8,   5  },
	{ 0b00001000,    8,   6  },
	{ 0b00000110,    8,   7  },
	{ 0b0000010011,  10, -10 },
	{ 0b0000010101,  10, -9  },
	{ 0b0000010111,  10, -8  },
	{ 0b0000010110,  10,  8  },
	{ 0b0000010100,  10,  9  },
	{ 0b0000010010,  10,  10 },
	{ 0b00000011001, 11, -16 },
	{ 0b00000011011, 11, -15 },
	{ 0b00000011101, 11, -14 },
	{ 0b00000011111, 11, -13 },
	{ 0b00000100001, 11, -12 },
	{ 0b00000100011, 11, -11 },
	{ 0b00000100010, 11,  11 },
	{ 0b00000100000, 11,  12 },
	{ 0b00000011110, 11,  13 },
	{ 0b00000011100, 11,  14 },
	{ 0b00000011010, 11,  15 },
	{ 0b00000011000, 11,  16 },
	{ 0, 0, 10 },
};

// for dmvector[t]
static const VLC_ENTRY vlc_entries_b11[] =
{
	{ 0b0,  1,  0 },
	{ 0b11, 2, -1 },
	{ 0b10, 2,  1 },
	{ 0, 0, 11 },
};

// dct_dc_size_luma
// checked (mpeg12data.c)
static const VLC_ENTRY vlc_entries_b12[] =
{
	{ 0b00,        2, 1 },
	{ 0b01,        2, 2 },
	{ 0b100,       3, 0 },
	{ 0b101,       3, 3 },
	{ 0b110,       3, 4 },
	{ 0b1110,      4, 5 },
	{ 0b11110,     5, 6 },
	{ 0b111110,    6, 7 },
	{ 0b1111110,   7, 8 },	// ISO11172-2 はここまで
	{ 0b11111110,  8, 9 },
	{ 0b111111110, 9, 10 },
	{ 0b111111111, 9, 11 },	// ISO13818-2 はここまで
	{ 0, 0, 12 },
};

// dct_dc_size_chroma
// checked (mpeg12data.c)
static const VLC_ENTRY vlc_entries_b13[] =
{
	{ 0b00,         2, 0 },
	{ 0b01,         2, 1 },
	{ 0b10,         2, 2 },
	{ 0b110,        3, 3 },
	{ 0b1110,       4, 4 },
	{ 0b11110,      5, 5 },
	{ 0b111110,     6, 6 },
	{ 0b1111110,    7, 7 },
	{ 0b11111110,   8, 8 },		// ISO11172-2 はここまで
	{ 0b111111110,  9, 9 },
	{ 0b1111111110, 10, 10 },
	{ 0b1111111111, 10, 11 },	// ISO13818-2 はここまで
	{ 0, 0, 13 },
};

// checked (mpeg12data.c)
static const VLC_ENTRY vlc_entries_b14[] =
{
	{ 0b10,                2, -1 },
	{ 0b11,                2, (0*256+1) },
	{ 0b011,               3, (1*256+1) },
	{ 0b0100,              4, (0*256+2) },
	{ 0b0101,              4, (2*256+1) },
	{ 0b00101,             5, (0*256+3) },
	{ 0b00111,             5, (3*256+1) },
	{ 0b00110,             5, (4*256+1) },
	{ 0b000110,            6, (1*256+2) },
	{ 0b000111,            6, (5*256+1) },
	{ 0b000101,            6, (6*256+1) },
	{ 0b000100,            6, (7*256+1) },
	{ 0b000001,            6, -9 },	// escape
	{ 0b0000110,           7, (0*256+4) },
	{ 0b0000100,           7, (2*256+2) },
	{ 0b0000111,           7, (8*256+1) },
	{ 0b0000101,           7, (9*256+1) },
	{ 0b00100110,          8, (0*256+5) },
	{ 0b00100001,          8, (0*256+6) },
	{ 0b00100101,          8, (1*256+3) },
	{ 0b00100100,          8, (3*256+2) },
	{ 0b00100111,          8, (10*256+1) },
	{ 0b00100011,          8, (11*256+1) },
	{ 0b00100010,          8, (12*256+1) },
	{ 0b00100000,          8, (13*256+1) },
	{ 0b0000001010,       10, (0*256+7) },
	{ 0b0000001100,       10, (1*256+4) },
	{ 0b0000001011,       10, (2*256+3) },
	{ 0b0000001111,       10, (4*256+2) },
	{ 0b0000001001,       10, (5*256+2) },
	{ 0b0000001110,       10, (14*256+1) },
	{ 0b0000001101,       10, (15*256+1) },
	{ 0b0000001000,       10, (16*256+1) },
	{ 0b000000011101,     12, (0*256+8) },
	{ 0b000000011000,     12, (0*256+9) },
	{ 0b000000010011,     12, (0*256+10) },
	{ 0b000000010000,     12, (0*256+11) },
	{ 0b000000011011,     12, (1*256+5) },
	{ 0b000000010100,     12, (2*256+4) },
	{ 0b000000011100,     12, (3*256+3) },
	{ 0b000000010010,     12, (4*256+3) },
	{ 0b000000011110,     12, (6*256+2) },
	{ 0b000000010101,     12, (7*256+2) },
	{ 0b000000010001,     12, (8*256+2) },
	{ 0b000000011111,     12, (17*256+1) },
	{ 0b000000011010,     12, (18*256+1) },
	{ 0b000000011001,     12, (19*256+1) },
	{ 0b000000010111,     12, (20*256+1) },
	{ 0b000000010110,     12, (21*256+1) },
	{ 0b0000000011010,    13, (0*256+12) },
	{ 0b0000000011001,    13, (0*256+13) },
	{ 0b0000000011000,    13, (0*256+14) },
	{ 0b0000000010111,    13, (0*256+15) },
	{ 0b0000000010110,    13, (1*256+6) },
	{ 0b0000000010101,    13, (1*256+7) },
	{ 0b0000000010100,    13, (2*256+5) },
	{ 0b0000000010011,    13, (3*256+4) },
	{ 0b0000000010010,    13, (5*256+3) },
	{ 0b0000000010001,    13, (9*256+2) },
	{ 0b0000000010000,    13, (10*256+2) },
	{ 0b0000000011111,    13, (22*256+1) },
	{ 0b0000000011110,    13, (23*256+1) },
	{ 0b0000000011101,    13, (24*256+1) },
	{ 0b0000000011100,    13, (25*256+1) },
	{ 0b0000000011011,    13, (26*256+1) },
	{ 0b00000000011111,   14, (0*256+16) },
	{ 0b00000000011110,   14, (0*256+17) },
	{ 0b00000000011101,   14, (0*256+18) },
	{ 0b00000000011100,   14, (0*256+19) },
	{ 0b00000000011011,   14, (0*256+20) },
	{ 0b00000000011010,   14, (0*256+21) },
	{ 0b00000000011001,   14, (0*256+22) },
	{ 0b00000000011000,   14, (0*256+23) },
	{ 0b00000000010111,   14, (0*256+24) },
	{ 0b00000000010110,   14, (0*256+25) },
	{ 0b00000000010101,   14, (0*256+26) },
	{ 0b00000000010100,   14, (0*256+27) },
	{ 0b00000000010011,   14, (0*256+28) },
	{ 0b00000000010010,   14, (0*256+29) },
	{ 0b00000000010001,   14, (0*256+30) },
	{ 0b00000000010000,   14, (0*256+31) },
	{ 0b000000000011000,  15, (0*256+32) },
	{ 0b000000000010111,  15, (0*256+33) },
	{ 0b000000000010110,  15, (0*256+34) },
	{ 0b000000000010101,  15, (0*256+35) },
	{ 0b000000000010100,  15, (0*256+36) },
	{ 0b000000000010011,  15, (0*256+37) },
	{ 0b000000000010010,  15, (0*256+38) },
	{ 0b000000000010001,  15, (0*256+39) },
	{ 0b000000000010000,  15, (0*256+40) },
	{ 0b000000000011111,  15, (1*256+8) },
	{ 0b000000000011110,  15, (1*256+9) },
	{ 0b000000000011101,  15, (1*256+10) },
	{ 0b000000000011100,  15, (1*256+11) },
	{ 0b000000000011011,  15, (1*256+12) },
	{ 0b000000000011010,  15, (1*256+13) },
	{ 0b000000000011001,  15, (1*256+14) },
	{ 0b0000000000010011, 16, (1*256+15) },
	{ 0b0000000000010010, 16, (1*256+16) },
	{ 0b0000000000010001, 16, (1*256+17) },
	{ 0b0000000000010000, 16, (1*256+18) },
	{ 0b0000000000010100, 16, (6*256+3) },
	{ 0b0000000000011010, 16, (11*256+2) },
	{ 0b0000000000011001, 16, (12*256+2) },
	{ 0b0000000000011000, 16, (13*256+2) },
	{ 0b0000000000010111, 16, (14*256+2) },
	{ 0b0000000000010110, 16, (15*256+2) },
	{ 0b0000000000010101, 16, (16*256+2) },
	{ 0b0000000000011111, 16, (27*256+1) },
	{ 0b0000000000011110, 16, (28*256+1) },
	{ 0b0000000000011101, 16, (29*256+1) },
	{ 0b0000000000011100, 16, (30*256+1) },
	{ 0b0000000000011011, 16, (31*256+1) },
	{ 0, 0, 14 },
};

// checked (mpeg12data.c)
static const VLC_ENTRY vlc_entries_b15[] =
{
	{ 0b10,                2, (0*256+1) },
	{ 0b010,               3, (1*256+1) },
	{ 0b110,               3, (0*256+2) },
	{ 0b0110,              4, -1 },	// end of block
	{ 0b0111,              4, (0*256+3) },
	{ 0b00101,             5, (2*256+1) },
	{ 0b00111,             5, (3*256+1) },
	{ 0b00110,             5, (1*256+2) },
	{ 0b11100,             5, (0*256+4) },
	{ 0b11101,             5, (0*256+5) },
	{ 0b000110,            6, (4*256+1) },
	{ 0b000111,            6, (5*256+1) },
	{ 0b000001,            6, -9 },	// escape
	{ 0b000101,            6, (0*256+6) },
	{ 0b000100,            6, (0*256+7) },
	{ 0b0000110,           7, (6*256+1) },
	{ 0b0000100,           7, (7*256+1) },
	{ 0b0000111,           7, (2*256+2) },
	{ 0b0000101,           7, (8*256+1) },
	{ 0b1111000,           7, (9*256+1) },
	{ 0b1111001,           7, (1*256+3) },
	{ 0b1111010,           7, (10*256+1) },
	{ 0b1111011,           7, (0*256+8) },
	{ 0b1111100,           7, (0*256+9) },
	{ 0b00100110,          8, (3*256+2) },
	{ 0b00100001,          8, (11*256+1) },
	{ 0b00100101,          8, (12*256+1) },
	{ 0b00100100,          8, (13*256+1) },
	{ 0b00100111,          8, (1*256+4) },
	{ 0b11111100,          8, (2*256+3) },
	{ 0b11111101,          8, (4*256+2) },
	{ 0b00100011,          8, (0*256+10) },
	{ 0b00100010,          8, (0*256+11) },
	{ 0b00100000,          8, (1*256+5) },
	{ 0b11111010,          8, (0*256+12) },
	{ 0b11111011,          8, (0*256+13) },
	{ 0b11111110,          8, (0*256+14) },
	{ 0b11111111,          8, (0*256+15) },
	{ 0b000000100,         9, (5*256+2) },
	{ 0b000000101,         9, (14*256+1) },
	{ 0b000000111,         9, (15*256+1) },
	{ 0b0000001101,       10, (16*256+1) },
	{ 0b0000001100,       10, (2*256+4) },
	{ 0b000000011100,     12, (3*256+3) },
	{ 0b000000010010,     12, (4*256+3) },
	{ 0b000000011110,     12, (6*256+2) },
	{ 0b000000010101,     12, (7*256+2) },
	{ 0b000000010001,     12, (8*256+2) },
	{ 0b000000011111,     12, (17*256+1) },
	{ 0b000000011010,     12, (18*256+1) },
	{ 0b000000011001,     12, (19*256+1) },
	{ 0b000000010111,     12, (20*256+1) },
	{ 0b000000010110,     12, (21*256+1) },
	{ 0b0000000010110,    13, (1*256+6) },
	{ 0b0000000010101,    13, (1*256+7) },
	{ 0b0000000010100,    13, (2*256+5) },
	{ 0b0000000010011,    13, (3*256+4) },
	{ 0b0000000010010,    13, (5*256+3) },
	{ 0b0000000010001,    13, (9*256+2) },
	{ 0b0000000010000,    13, (10*256+2) },
	{ 0b0000000011111,    13, (22*256+1) },
	{ 0b0000000011110,    13, (23*256+1) },
	{ 0b0000000011101,    13, (24*256+1) },
	{ 0b0000000011100,    13, (25*256+1) },
	{ 0b0000000011011,    13, (26*256+1) },
	{ 0b00000000011111,   14, (0*256+16) },
	{ 0b00000000011110,   14, (0*256+17) },
	{ 0b00000000011101,   14, (0*256+18) },
	{ 0b00000000011100,   14, (0*256+19) },
	{ 0b00000000011011,   14, (0*256+20) },
	{ 0b00000000011010,   14, (0*256+21) },
	{ 0b00000000011001,   14, (0*256+22) },
	{ 0b00000000011000,   14, (0*256+23) },
	{ 0b00000000010111,   14, (0*256+24) },
	{ 0b00000000010110,   14, (0*256+25) },
	{ 0b00000000010101,   14, (0*256+26) },
	{ 0b00000000010100,   14, (0*256+27) },
	{ 0b00000000010011,   14, (0*256+28) },
	{ 0b00000000010010,   14, (0*256+29) },
	{ 0b00000000010001,   14, (0*256+30) },
	{ 0b00000000010000,   14, (0*256+31) },
	{ 0b000000000011000,  15, (0*256+32) },
	{ 0b000000000010111,  15, (0*256+33) },
	{ 0b000000000010110,  15, (0*256+34) },
	{ 0b000000000010101,  15, (0*256+35) },
	{ 0b000000000010100,  15, (0*256+36) },
	{ 0b000000000010011,  15, (0*256+37) },
	{ 0b000000000010010,  15, (0*256+38) },
	{ 0b000000000010001,  15, (0*256+39) },
	{ 0b000000000010000,  15, (0*256+40) },
	{ 0b000000000011111,  15, (1*256+8) },
	{ 0b000000000011110,  15, (1*256+9) },
	{ 0b000000000011101,  15, (1*256+10) },
	{ 0b000000000011100,  15, (1*256+11) },
	{ 0b000000000011011,  15, (1*256+12) },
	{ 0b000000000011010,  15, (1*256+13) },
	{ 0b000000000011001,  15, (1*256+14) },
	{ 0b0000000000010011, 16, (1*256+15) },
	{ 0b0000000000010010, 16, (1*256+16) },
	{ 0b0000000000010001, 16, (1*256+17) },
	{ 0b0000000000010000, 16, (1*256+18) },
	{ 0b0000000000010100, 16, (6*256+3) },
	{ 0b0000000000011010, 16, (11*256+2) },
	{ 0b0000000000011001, 16, (12*256+2) },
	{ 0b0000000000011000, 16, (13*256+2) },
	{ 0b0000000000010111, 16, (14*256+2) },
	{ 0b0000000000010110, 16, (15*256+2) },
	{ 0b0000000000010101, 16, (16*256+2) },
	{ 0b0000000000011111, 16, (27*256+1) },
	{ 0b0000000000011110, 16, (28*256+1) },
	{ 0b0000000000011101, 16, (29*256+1) },
	{ 0b0000000000011100, 16, (30*256+1) },
	{ 0b0000000000011011, 16, (31*256+1) },
	{ 0, 0, 15 },
};

VLC_TABLE vlc_table_b1;
VLC_TABLE vlc_table_b2;
VLC_TABLE vlc_table_b3;
VLC_TABLE vlc_table_b4;
VLC_TABLE vlc_table_b9;
VLC_TABLE vlc_table_b10;
VLC_TABLE vlc_table_b11;
VLC_TABLE vlc_table_b12;
VLC_TABLE vlc_table_b13;
VLC_TABLE vlc_table_b14;
VLC_TABLE vlc_table_b15;

////////////////////////////////////////////////////////////////////////////////
/// @brief VLCデコード用のテーブルを作成する
///
/// エントリ列から、1発索引用の巨大テーブルを構築する。
///
static int vlc_make_table(VLC_TABLE* t, const VLC_ENTRY* e)
{
	t->bits = 0;

	// まず最長コード数を得る
	for(int i = 0; e[i].clen; ++i) if(t->bits < e[i].clen) t->bits = e[i].clen;

	int items = 1 << t->bits;

	// メモリ確保
	if(!(t->table = (int*)malloc(sizeof(int) * items * 2)))
	{
		printf("Error: Not enough memory (at vlc_make_table)\n");
		return 0;
	}

	// とりあえずエラー(-99)で埋めとく TODO:INT_MINのほうが良くなイカ？
	for(int i = 0; i < items; ++i)
	{
		t->table[i*2+0] = -99;
		t->table[i*2+1] = 0;
	}

	// 一個ずつテーブルに書き込み
	for(; e->clen; ++e)
	{
		int res_bits = t->bits - e->clen;
		int code = e->code << res_bits;
		for(int i = (1 << res_bits); i > 0; --i, ++code)
		{
			t->table[code*2+0] = e->value;
			t->table[code*2+1] = e->clen;
		}
	}

	return 1;
}

////////////////////////////////////////////////////////////////////////////////
/// @brief VLCデコード用のテーブルを作成する
///
/// 各エントリ列について、テーブルの作成を行う
///
int vlc_init_table()
{
	if(!vlc_make_table(&vlc_table_b1,  vlc_entries_b1 )) return 0;
	if(!vlc_make_table(&vlc_table_b2,  vlc_entries_b2 )) return 0;
	if(!vlc_make_table(&vlc_table_b3,  vlc_entries_b3 )) return 0;
	if(!vlc_make_table(&vlc_table_b4,  vlc_entries_b4 )) return 0;
	if(!vlc_make_table(&vlc_table_b9,  vlc_entries_b9 )) return 0;
	if(!vlc_make_table(&vlc_table_b10, vlc_entries_b10)) return 0;
	if(!vlc_make_table(&vlc_table_b11, vlc_entries_b11)) return 0;
	if(!vlc_make_table(&vlc_table_b12, vlc_entries_b12)) return 0;
	if(!vlc_make_table(&vlc_table_b13, vlc_entries_b13)) return 0;
	if(!vlc_make_table(&vlc_table_b14, vlc_entries_b14)) return 0;
	if(!vlc_make_table(&vlc_table_b15, vlc_entries_b15)) return 0;
	return 1;
}

////////////////////////////////////////////////////////////////////////////////
/// @brief VLCデコード用のテーブルを解放する
///
void vlc_free()
{
	free(vlc_table_b1.table);  vlc_table_b1.table = NULL;
	free(vlc_table_b2.table);  vlc_table_b2.table = NULL;
	free(vlc_table_b3.table);  vlc_table_b3.table = NULL;
	free(vlc_table_b4.table);  vlc_table_b4.table = NULL;
	free(vlc_table_b9.table);  vlc_table_b9.table = NULL;
	free(vlc_table_b10.table); vlc_table_b10.table = NULL;
	free(vlc_table_b11.table); vlc_table_b11.table = NULL;
	free(vlc_table_b12.table); vlc_table_b12.table = NULL;
	free(vlc_table_b13.table); vlc_table_b13.table = NULL;
	free(vlc_table_b14.table); vlc_table_b14.table = NULL;
	free(vlc_table_b15.table); vlc_table_b15.table = NULL;
}

